<?php

/**
 * Common include used in all SureInvoice pages
 *
 * @version $Id$
 * @copyright 2003 
 **/
require_once('DBConn.php');
require_once('global_config.php');
require_once('html.php');
require_once('User.php');
require_once('Config.php');
session_start();

ob_start();

$db_conn = new DBConn(DB_SERVER, DB_DATABASE, DB_USER, DB_PASSWORD, TRUE);
if($db_conn->connect() == FALSE){
	trigger_error("Could not connect to database!\n".$db_conn->getLastError(), E_USER_ERROR);
}

$GLOBALS['CONFIG'] = Config::getAppConfig();


// Error handling functions
function site_error_handler($errno, $errstr, $errfile, $errline){
	$today = date("D M j G:i:s T Y");

	if(isset($GLOBALS['CONFIG']['error_log']) && is_file($GLOBALS['CONFIG']['error_log']) && is_writeable($GLOBALS['CONFIG']['error_log'])){
		$error_log_avail = TRUE;
	}else{
		error_log("sure_invoice: Could not write to error log: ".$GLOBALS['CONFIG']['error_log'], 0);
		$error_log_avail = FALSE;
	}

	// Log the error
	if($error_log_avail && $errno != E_NOTICE)
		error_log("$today: PHP Error ($errno): $errstr in $errfile at $errline.\n", 3, $GLOBALS['CONFIG']['error_log']);

	// Email error
	if(isset($GLOBALS['CONFIG']['error_email']) && $errno != E_NOTICE && $errno != E_USER_NOTICE)
		error_log("{$_SERVER['SERVER_NAME']} Server Error\n\n$today: PHP Error ($errno): $errstr in $errfile at $errline.\n", 1, $APP_GLOBALS['ERROR_EMAIL']);

	// If it is an error redirect to error page or print message
	if($errno == E_COMPILE_ERROR || $errno == E_CORE_ERROR || $errno == E_USER_ERROR || $errno == E_ERROR){
		if($GLOBALS['CONFIG']['debug']){
			$error_msg = "<PRE>PHP Error ($errno): $errfile at $errline.\n\n$errstr\n\n\n".format_backtrace(debug_backtrace())."\n\n</PRE>";
		}else{
			$error_msg = "We are sorry, an error has occured while processing your request, please try again later.\n".
				"The system administrator has been notified of the problem.\n";
		}
		fatal_error($error_msg);
	}
}

function fatal_error($message){

	//print($message);
	header("Location: ".$GLOBALS['CONFIG']['url']."error.php?error=".urlencode($message)."\r\n");
	exit();

}

function format_backtrace($backtrace){
	$output = "<b>Debug Backtrace:\n</b>";
	if(is_array($backtrace)){
		foreach($backtrace as $item){
			if(isset($item['file']))
				$output .= $item['file'];
			if(isset($item['line']))
				$output .= " at ".$item['line'];
			$output .= "\n";
		}
	}
	return $output;
}

set_error_handler(site_error_handler);


// Authentication functions
$loggedin_user =& $_SESSION['userObj'];

function loginUser($email, $password){
	$user = User::getUserByLogin($email, $password);
	if($user === FALSE){
		unset($_SESSION['userObj']);
		return FALSE;
	}else{
		$user->hasRight("admin");
		$_SESSION['userObj'] = $user;
		return TRUE;
	}
}

function isLoggedIn(){
	global $_SESSION;

	if(!isset($_SESSION['userObj'])){
		return FALSE;
	}

	if(get_class($_SESSION['userObj']) == "user"){
		return TRUE;
	}else{
		return FALSE;
	}
}


function checkLogin($section = ''){
	global $loggedin_user;

	if(!isLoggedIn()){
		header("Location: login.php?referrer=".urlencode($_SERVER['PHP_SELF']."?".$_SERVER['QUERY_STRING']));
		exit();
	}else{
		if($section == "")
			return TRUE;
		else{
			if($loggedin_user->hasRight($section))
				return TRUE;
			else{
				trigger_error("Insufficent Access Rights, you must have the $section right to access this page!\n", E_USER_ERROR);
				exit();
			}
		}
	}
}

function debug_message($message){
	global $error_msg;
	$today = date("D M j G:i:s T Y");
	
	if($GLOBALS['CONFIG']['debug'] > 0){
		$error_msg .= "DEBUG: ".$message."\n";
		trigger_error($message, E_USER_ERROR);
	}
	
	if(isset($GLOBALS['CONFIG']['error_log']) && is_file($GLOBALS['CONFIG']['error_log']) && is_writeable($GLOBALS['CONFIG']['error_log'])){
		$error_log_avail = TRUE;
	}else{
		error_log("sure_invoice: Could not write to error log: ".$GLOBALS['CONFIG']['error_log'], 0);
		$error_log_avail = FALSE;		
	}	
	
	if($error_log_avail == TRUE){
		error_log("[$today]:[".$_SERVER['PHP_SELF']."]:\n".$message, 3, $GLOBALS['CONFIG']['error_log']);
	}
}

// Make sure we have some key vars setup
if(!is_dir($GLOBALS['CONFIG']['attachment_dir']) && !empty($GLOBALS['CONFIG']['attachment_dir'])) {
	fatal_error("attachment_dir {$GLOBALS['CONFIG']['attachment_dir']} is not directory!");
}

if(substr($GLOBALS['CONFIG']['attachment_dir'], -1, 1) != '/'){
	$GLOBALS['CONFIG']['attachment_dir'] .= '/';
}

function goBack($num = 1){
	$num = $num + 1; //Pad number to avoid redirecting to self
	if(count($_SESSION['history'] >= $num)){
		$location = '';
		for($i = 0; $i < $num; $i++){
			$location = array_pop($_SESSION['history']);
		}
		header("Location: ".$location."\r\n");
	}elseif(isLoggedIn()){
		header("Location: ".$_SESSION['userObj']->start_page."\r\n");
	}else{
		header("Location: index.php\r\n");
	}
	exit();
}


// Use the session to determine redirects correctly
// Add current URL to history if it is a GET and not a duplicate of the last URL
if($_SERVER['REQUEST_METHOD'] == 'GET' &&
	 $_SERVER['PHP_SELF'].'?'.$_SERVER['QUERY_STRING'] != $_SESSION['history'][count($_SESSION['history'])-1]){
	$_SESSION['history'][] = $_SERVER['PHP_SELF'].'?'.$_SERVER['QUERY_STRING'];
}

// Keep history array small
while(count($_SESSION['history'])>10){
	array_shift($_SESSION['history']);
}
?>
